@@ -25,7 +25,6 @@ import os_net_config
 from os_net_config import objects
 from os_net_config import utils
 
-
 logger = logging.getLogger(__name__)
 
 # Import the raw NetConfig object so we can call its methods
@@ -134,6 +133,7 @@ class IfcfgNetConfig(os_net_config.NetConfig):
         self.nfvswitch_intiface_data = {}
         self.nfvswitch_options = None
         self.vlan_data = {}
+        self.ib_childs_data = {}
         self.route_data = {}
         self.route6_data = {}
         self.route_table_data = {}
@@ -170,6 +170,14 @@ class IfcfgNetConfig(os_net_config.NetConfig):
                 routes.append(line)
         return routes
 
+    def parse_ifcfg_rules(self, ifcfg_data):
+        """Break out the individual rules from an ifcfg rule file."""
+        rules = []
+        for line in ifcfg_data.split("\n"):
+            if not line.startswith("#"):
+                rules.append(line)
+        return rules
+
     def enumerate_ifcfg_changes(self, ifcfg_data_old, ifcfg_data_new):
         """Determine which values are added/modified/removed
 
@@ -208,6 +216,24 @@ class IfcfgNetConfig(os_net_config.NetConfig):
                 route_changes.append((route, 'added'))
         return route_changes
 
+    def enumerate_ifcfg_rule_changes(self, old_rules, new_rules):
+        """Determine which routes are added or removed.
+
+        :param file_values: contents of existing interface route rule file
+        :param data_values: contents of replacement interface route rule file
+        :return: list of tuples representing changes (rule, state), where
+                 state is one of added or removed
+        """
+
+        rule_changes = []
+        for rule in old_rules:
+            if rule not in new_rules:
+                rule_changes.append((rule, 'removed'))
+        for rule in new_rules:
+            if rule not in old_rules:
+                rule_changes.append((rule, 'added'))
+        return rule_changes
+
     def ifcfg_requires_restart(self, filename, new_data):
         """Determine if changes to the ifcfg file require a restart to apply.
 
@@ -327,6 +353,31 @@ class IfcfgNetConfig(os_net_config.NetConfig):
                 commands.append('route add ' + route[0])
         return commands
 
+    def iproute2_rule_commands(self, filename, data):
+        """Return a list of commands for 'ip route' to modify routing rules.
+
+           The list of commands is generated by comparing the old and new
+           configs, and calculating which rules need to be added and which
+           need to be removed.
+
+        :param filename: path to the original interface route rule file
+        :param data: data that is to be written to new route rule file
+        :return: list of commands to feed to 'ip' to reconfigure route rules
+        """
+
+        file_values = self.parse_ifcfg_rules(utils.get_file_data(filename))
+        data_values = self.parse_ifcfg_rules(data)
+        rule_changes = self.enumerate_ifcfg_rule_changes(file_values,
+                                                         data_values)
+        commands = []
+
+        for rule in rule_changes:
+            if rule[1] == 'removed':
+                commands.append('rule del ' + rule[0])
+            elif rule[1] == 'added':
+                commands.append('rule add ' + rule[0])
+        return commands
+
     def child_members(self, name):
         children = set()
         try:
@@ -374,8 +425,16 @@ class IfcfgNetConfig(os_net_config.NetConfig):
             data += "TYPE=NFVSWITCHIntPort\n"
         elif isinstance(base_opt, objects.IbInterface):
             data += "TYPE=Infiniband\n"
+        elif isinstance(base_opt, objects.IbChildInterface):
+            data += "TYPE=Infiniband\n"
+            data += "PKEY=yes\n"
+            data += "PHYSDEV=%s\n" % base_opt.parent
+            data += "PKEY_ID=%s\n" % base_opt.pkey_id
         elif re.match('\w+\.\d+$', base_opt.name):
             data += "VLAN=yes\n"
+        elif isinstance(base_opt, objects.Interface):
+            if base_opt.linkdelay:
+                data += "LINKDELAY=%s\n" % base_opt.linkdelay
         if base_opt.linux_bond_name:
             data += "MASTER=%s\n" % base_opt.linux_bond_name
             data += "SLAVE=yes\n"
@@ -750,7 +809,7 @@ class IfcfgNetConfig(os_net_config.NetConfig):
         self.bridge_data[bridge.name] = data
         if bridge.routes:
             self._add_routes(bridge.name, bridge.routes)
-        if bridge.routes:
+        if bridge.rules:
             self._add_rules(bridge.name, bridge.rules)
 
     def add_ovs_user_bridge(self, bridge):
@@ -884,6 +943,22 @@ class IfcfgNetConfig(os_net_config.NetConfig):
                         % (ib_interface.hwname, ib_interface.name))
             self.renamed_interfaces[ib_interface.hwname] = ib_interface.name
 
+    def add_ib_child_interface(self, ib_child_interface):
+        """Add an InfiniBand child interface object to the net config object.
+
+        :param ib_child_interface: The InfiniBand child
+         interface object to add.
+        """
+        logger.info('adding ib_child_interface: %s' % ib_child_interface.name)
+        data = self._add_common(ib_child_interface)
+        logger.debug('ib_child_interface data: %s' % data)
+        self.ib_childs_data[ib_child_interface.name] = data
+        if ib_child_interface.routes:
+            self._add_routes(ib_child_interface.name,
+                             ib_child_interface.routes)
+        if ib_child_interface.rules:
+            self._add_rules(ib_child_interface.name, ib_child_interface.rules)
+
     def add_ovs_dpdk_port(self, ovs_dpdk_port):
         """Add a OvsDpdkPort object to the net config object.
 
@@ -940,6 +1015,10 @@ class IfcfgNetConfig(os_net_config.NetConfig):
                                   self.noop, promisc=sriov_pf.promisc,
                                   link_mode=sriov_pf.link_mode)
         self.interface_data[sriov_pf.name] = data
+        if sriov_pf.routes:
+            self._add_routes(sriov_pf.name, sriov_pf.routes)
+        if sriov_pf.rules:
+            self._add_rules(sriov_pf.name, sriov_pf.rules)
 
     def add_sriov_vf(self, sriov_vf):
         """Add a SriovVF object to the net config object
@@ -1146,6 +1225,7 @@ class IfcfgNetConfig(os_net_config.NetConfig):
         logger.info('applying network configs...')
         restart_interfaces = []
         restart_vlans = []
+        restart_ib_childs = []
         restart_bridges = []
         restart_linux_bonds = []
         restart_linux_teams = []
@@ -1153,6 +1233,7 @@ class IfcfgNetConfig(os_net_config.NetConfig):
         apply_interfaces = []
         apply_bridges = []
         apply_routes = []
+        apply_rules = []
         update_files = {}
         all_file_names = []
         ivs_uplinks = []  # ivs physical uplinks
@@ -1207,6 +1288,8 @@ class IfcfgNetConfig(os_net_config.NetConfig):
                     apply_routes.append((interface_name, route6_data))
             if utils.diff(rule_path, rule_data):
                 update_files[rule_path] = rule_data
+                if interface_name not in restart_interfaces:
+                    apply_rules.append((interface_name, rule_data))
 
         for interface_name, iface_data in self.ivsinterface_data.items():
             route_data = self.route_data.get(interface_name, '')
@@ -1241,6 +1324,8 @@ class IfcfgNetConfig(os_net_config.NetConfig):
                     apply_routes.append((interface_name, route6_data))
             if utils.diff(rule_path, rule_data):
                 update_files[rule_path] = rule_data
+                if interface_name not in restart_interfaces:
+                    apply_rules.append((interface_name, rule_data))
 
         for iface_name, iface_data in self.nfvswitch_intiface_data.items():
             route_data = self.route_data.get(iface_name, '')
@@ -1275,6 +1360,8 @@ class IfcfgNetConfig(os_net_config.NetConfig):
                     apply_routes.append((iface_name, route6_data))
             if utils.diff(rule_path, rule_data):
                 update_files[rule_path] = rule_data
+                if iface_name not in restart_interfaces:
+                    apply_rules.append((iface_name, rule_data))
 
         for bridge_name, bridge_data in self.bridge_data.items():
             route_data = self.route_data.get(bridge_name, '')
@@ -1312,6 +1399,8 @@ class IfcfgNetConfig(os_net_config.NetConfig):
                     apply_routes.append((bridge_name, route6_data))
             if utils.diff(br_rule_path, rule_data):
                 update_files[br_rule_path] = rule_data
+                if bridge_name not in restart_interfaces:
+                    apply_rules.append((bridge_name, rule_data))
 
         for bridge_name, bridge_data in self.linuxbridge_data.items():
             route_data = self.route_data.get(bridge_name, '')
@@ -1349,6 +1438,8 @@ class IfcfgNetConfig(os_net_config.NetConfig):
                     apply_routes.append((bridge_name, route6_data))
             if utils.diff(br_rule_path, rule_data):
                 update_files[br_rule_path] = rule_data
+                if bridge_name not in restart_bridges:
+                    apply_rules.append((bridge_name, rule_data))
 
         for team_name, team_data in self.linuxteam_data.items():
             route_data = self.route_data.get(team_name, '')
@@ -1462,6 +1553,8 @@ class IfcfgNetConfig(os_net_config.NetConfig):
                     apply_routes.append((interface_name, route6_data))
             if utils.diff(rule_path, rule_data):
                 update_files[rule_path] = rule_data
+                if interface_name not in restart_interfaces:
+                    apply_rules.append((interface_name, rule_data))
 
         # NOTE(hjensas): Process the VLAN's last so that we know if the vlan's
         # parent interface is being restarted.
@@ -1506,6 +1599,55 @@ class IfcfgNetConfig(os_net_config.NetConfig):
                     apply_routes.append((vlan_name, route6_data))
             if utils.diff(vlan_rule_path, rule_data):
                 update_files[vlan_rule_path] = rule_data
+                if vlan_name not in restart_vlans:
+                    apply_rules.append((vlan_name, rule_data))
+
+        for ib_child_name, ib_child_data in self.ib_childs_data.items():
+            route_data = self.route_data.get(ib_child_name, '')
+            route6_data = self.route6_data.get(ib_child_name, '')
+            rule_data = self.rule_data.get(ib_child_name, '')
+            ib_child_path = self.root_dir + ifcfg_config_path(ib_child_name)
+            ib_child_route_path = \
+                self.root_dir + route_config_path(ib_child_name)
+            ib_child_route6_path = \
+                self.root_dir + route6_config_path(ib_child_name)
+            ib_child_rule_path = \
+                self.root_dir + route_rule_config_path(ib_child_name)
+            all_file_names.append(ib_child_path)
+            all_file_names.append(ib_child_route_path)
+            all_file_names.append(ib_child_route6_path)
+            all_file_names.append(ib_child_rule_path)
+            restarts_concatenated = itertools.chain(restart_interfaces,
+                                                    restart_bridges,
+                                                    restart_linux_bonds,
+                                                    restart_linux_teams)
+            if (self.parse_ifcfg(ib_child_data).get('PHYSDEV') in
+                    restarts_concatenated):
+                if ib_child_name not in restart_ib_childs:
+                    restart_ib_childs.append(ib_child_name)
+                update_files[ib_child_path] = ib_child_data
+            elif utils.diff(ib_child_path, ib_child_data):
+                if self.ifcfg_requires_restart(ib_child_path, ib_child_data):
+                    restart_ib_childs.append(ib_child_name)
+                else:
+                    apply_interfaces.append(
+                        (ib_child_name, ib_child_path, ib_child_data))
+                update_files[ib_child_path] = ib_child_data
+            else:
+                logger.info('No changes required for the ib child interface: '
+                            '%s' % ib_child_name)
+            if utils.diff(ib_child_route_path, route_data):
+                update_files[ib_child_route_path] = route_data
+                if ib_child_name not in restart_ib_childs:
+                    apply_routes.append((ib_child_name, route_data))
+            if utils.diff(ib_child_route6_path, route6_data):
+                update_files[ib_child_route6_path] = route6_data
+                if ib_child_name not in restart_ib_childs:
+                    apply_routes.append((ib_child_name, route6_data))
+            if utils.diff(ib_child_rule_path, rule_data):
+                update_files[ib_child_rule_path] = rule_data
+                if ib_child_name not in restart_ib_childs:
+                    apply_rules.append((ib_child_name, rule_data))
 
         if self.vpp_interface_data or self.vpp_bond_data:
             vpp_path = self.root_dir + vpp_config_path()
@@ -1582,9 +1724,30 @@ class IfcfgNetConfig(os_net_config.NetConfig):
                             self.child_members(interface[0]))
                         break
 
+            for interface in apply_rules:
+                logger.debug('Applying rules for interface %s' % interface[0])
+                filename = self.root_dir + route_rule_config_path(interface[0])
+                commands = self.iproute2_rule_commands(filename, interface[1])
+                for command in commands:
+                    args = command.split()
+                    try:
+                        if len(args) > 0:
+                            self.execute('Running ip %s' % command, ipcmd,
+                                         *args)
+                    except Exception as e:
+                        logger.warning("Error in 'ip %s', restarting %s:\n%s" %
+                                       (command, interface[0], str(e)))
+                        restart_interfaces.append(interface[0])
+                        restart_interfaces.extend(
+                            self.child_members(interface[0]))
+                        break
+
             for vlan in restart_vlans:
                 self.ifdown(vlan)
 
+            for ib_child in restart_ib_childs:
+                self.ifdown(ib_child)
+
             for interface in restart_interfaces:
                 self.ifdown(interface)
 
@@ -1684,6 +1847,9 @@ class IfcfgNetConfig(os_net_config.NetConfig):
                 for nfvswitch_internal in nfvswitch_internal_ifaces:
                     self.ifup(nfvswitch_internal)
 
+            for ib_child in restart_ib_childs:
+                self.ifup(ib_child)
+
             for vlan in restart_vlans:
                 self.ifup(vlan)
 
