@@ -14,12 +14,12 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
-import mock
 import os
 import os.path
 import random
 import shutil
 import tempfile
+from unittest import mock
 import yaml
 
 from os_net_config import objects
@@ -119,6 +119,10 @@ class TestUtils(base.TestCase):
         shutil.rmtree(tmpdir)
 
     def test_update_sriov_pf_map_new(self):
+        def get_numvfs_stub(pf_name):
+            return 0
+        self.stub_out('os_net_config.sriov_config.get_numvfs',
+                      get_numvfs_stub)
         utils.update_sriov_pf_map('eth1', 10, False)
         contents = utils.get_file_data(sriov_config._SRIOV_CONFIG_FILE)
         sriov_pf_map = yaml.safe_load(contents) if contents else []
@@ -127,7 +131,32 @@ class TestUtils(base.TestCase):
                               'name': 'eth1', 'numvfs': 10}]
         self.assertListEqual(test_sriov_pf_map, sriov_pf_map)
 
+    def test_update_sriov_pf_map_with_same_numvfs(self):
+        def get_numvfs_stub(pf_name):
+            return 10
+        self.stub_out('os_net_config.sriov_config.get_numvfs',
+                      get_numvfs_stub)
+        utils.update_sriov_pf_map('eth1', 10, False)
+        contents = utils.get_file_data(sriov_config._SRIOV_CONFIG_FILE)
+        sriov_pf_map = yaml.safe_load(contents) if contents else []
+        self.assertEqual(1, len(sriov_pf_map))
+        test_sriov_pf_map = [{'device_type': 'pf', 'link_mode': 'legacy',
+                              'name': 'eth1', 'numvfs': 10}]
+        self.assertListEqual(test_sriov_pf_map, sriov_pf_map)
+
+    def test_update_sriov_pf_map_with_diff_numvfs(self):
+        def get_numvfs_stub(pf_name):
+            return 12
+        self.stub_out('os_net_config.sriov_config.get_numvfs',
+                      get_numvfs_stub)
+        self.assertRaises(sriov_config.SRIOVNumvfsException,
+                          utils.update_sriov_pf_map, 'eth1', 10, False)
+
     def test_update_sriov_pf_map_new_with_promisc(self):
+        def get_numvfs_stub(pf_name):
+            return 0
+        self.stub_out('os_net_config.sriov_config.get_numvfs',
+                      get_numvfs_stub)
         utils.update_sriov_pf_map('eth1', 10, False, promisc='off')
         contents = utils.get_file_data(sriov_config._SRIOV_CONFIG_FILE)
         sriov_pf_map = yaml.safe_load(contents) if contents else []
@@ -137,27 +166,28 @@ class TestUtils(base.TestCase):
         self.assertListEqual(test_sriov_pf_map, sriov_pf_map)
 
     def test_update_sriov_pf_map_exist(self):
+        def get_numvfs_stub(pf_name):
+            return 10
+        self.stub_out('os_net_config.sriov_config.get_numvfs',
+                      get_numvfs_stub)
         pf_initial = [{'device_type': 'pf', 'link_mode': 'legacy',
                        'name': 'eth1', 'numvfs': 10}]
         utils.write_yaml_config(sriov_config._SRIOV_CONFIG_FILE, pf_initial)
-
-        utils.update_sriov_pf_map('eth1', 20, False)
-        pf_final = [{'device_type': 'pf', 'link_mode': 'legacy',
-                     'name': 'eth1', 'numvfs': 20}]
-        contents = utils.get_file_data(sriov_config._SRIOV_CONFIG_FILE)
-
-        pf_map = yaml.safe_load(contents) if contents else []
-        self.assertEqual(1, len(pf_map))
-        self.assertListEqual(pf_final, pf_map)
+        self.assertRaises(sriov_config.SRIOVNumvfsException,
+                          utils.update_sriov_pf_map, 'eth1', 20, False)
 
     def test_update_sriov_pf_map_exist_with_promisc(self):
+        def get_numvfs_stub(pf_name):
+            return 10
+        self.stub_out('os_net_config.sriov_config.get_numvfs',
+                      get_numvfs_stub)
         pf_initial = [{'device_type': 'pf', 'link_mode': 'legacy',
                        'name': 'eth1', 'numvfs': 10, 'promisc': 'on'}]
         utils.write_yaml_config(sriov_config._SRIOV_CONFIG_FILE, pf_initial)
 
-        utils.update_sriov_pf_map('eth1', 20, False)
+        utils.update_sriov_pf_map('eth1', 10, False, promisc='off')
         pf_final = [{'device_type': 'pf', 'link_mode': 'legacy',
-                     'name': 'eth1', 'numvfs': 20, 'promisc': 'on'}]
+                     'name': 'eth1', 'numvfs': 10, 'promisc': 'off'}]
         contents = utils.get_file_data(sriov_config._SRIOV_CONFIG_FILE)
 
         pf_map = yaml.safe_load(contents) if contents else []
@@ -170,23 +200,27 @@ class TestUtils(base.TestCase):
         sriov_vf_map = yaml.safe_load(contents) if contents else []
         self.assertEqual(1, len(sriov_vf_map))
         test_sriov_vf_map = [{'device_type': 'vf', 'name': 'eth1_2',
+                              'min_tx_rate': 0, 'max_tx_rate': 0,
                               'device': {"name": "eth1", "vfid": 2}}]
         self.assertListEqual(test_sriov_vf_map, sriov_vf_map)
 
     def test_update_sriov_vf_map_complete_new(self):
         utils.update_sriov_vf_map('eth1', 2, 'eth1_2', vlan_id=10, qos=5,
                                   spoofcheck="on", trust="on", state="enable",
-                                  macaddr="AA:BB:CC:DD:EE:FF", promisc="off")
+                                  macaddr="AA:BB:CC:DD:EE:FF", promisc="off",
+                                  pci_address="0000:80:00.1", max_tx_rate=10)
         contents = utils.get_file_data(sriov_config._SRIOV_CONFIG_FILE)
         sriov_vf_map = yaml.safe_load(contents) if contents else []
         self.assertEqual(1, len(sriov_vf_map))
         test_sriov_vf_map = [{'device_type': 'vf', 'name': 'eth1_2',
                               'device': {'name': 'eth1', 'vfid': 2},
                               'vlan_id': 10, 'qos': 5,
+                              'min_tx_rate': 0, 'max_tx_rate': 10,
                               'spoofcheck': 'on', 'trust': 'on',
                               'state': 'enable',
                               'macaddr': 'AA:BB:CC:DD:EE:FF',
-                              'promisc': 'off'}]
+                              'promisc': 'off',
+                              'pci_address': "0000:80:00.1"}]
         self.assertListEqual(test_sriov_vf_map, sriov_vf_map)
 
     def test_update_sriov_vf_map_exist(self):
@@ -196,14 +230,17 @@ class TestUtils(base.TestCase):
 
         utils.update_sriov_vf_map('eth1', 2, 'eth1_2', vlan_id=10, qos=5,
                                   spoofcheck="on", trust="on", state="enable",
-                                  macaddr="AA:BB:CC:DD:EE:FF", promisc="off")
+                                  macaddr="AA:BB:CC:DD:EE:FF", promisc="off",
+                                  pci_address="0000:80:00.1", max_tx_rate=10)
         vf_final = [{'device_type': 'vf', 'name': 'eth1_2',
                      'device': {'name': 'eth1', 'vfid': 2},
                      'vlan_id': 10, 'qos': 5,
+                     'min_tx_rate': 0, 'max_tx_rate': 10,
                      'spoofcheck': 'on', 'trust': 'on',
                      'state': 'enable',
                      'macaddr': 'AA:BB:CC:DD:EE:FF',
-                     'promisc': 'off'}]
+                     'promisc': 'off',
+                     'pci_address': '0000:80:00.1'}]
         contents = utils.get_file_data(sriov_config._SRIOV_CONFIG_FILE)
 
         vf_map = yaml.safe_load(contents) if contents else []
@@ -214,22 +251,27 @@ class TestUtils(base.TestCase):
         vf_initial = [{'device_type': 'vf', 'name': 'eth1_2',
                        'device': {'name': 'eth1', 'vfid': 2},
                        'vlan_id': 10, 'qos': 5,
+                       'min_tx_rate': 0, 'max_tx_rate': 00,
                        'spoofcheck': 'on', 'trust': 'on',
                        'state': 'enable',
                        'macaddr': 'AA:BB:CC:DD:EE:FF',
-                       'promisc': 'off'}]
+                       'promisc': 'off',
+                       'pci_address': "0000:80:00.1"}]
         utils.write_yaml_config(sriov_config._SRIOV_CONFIG_FILE, vf_initial)
 
         utils.update_sriov_vf_map('eth1', 2, 'eth1_2', vlan_id=100, qos=15,
                                   spoofcheck="off", trust="off", state="auto",
-                                  macaddr="BB:BB:CC:DD:EE:FF", promisc="on")
+                                  macaddr="BB:BB:CC:DD:EE:FF", promisc="on",
+                                  pci_address="0000:80:00.1", max_tx_rate=40)
         vf_final = [{'device_type': 'vf', 'name': 'eth1_2',
                      'device': {'name': 'eth1', 'vfid': 2},
                      'vlan_id': 100, 'qos': 15,
+                     'min_tx_rate': 0, 'max_tx_rate': 40,
                      'spoofcheck': 'off', 'trust': 'off',
                      'state': 'auto',
                      'macaddr': 'BB:BB:CC:DD:EE:FF',
-                     'promisc': 'on'}]
+                     'promisc': 'on',
+                     'pci_address': '0000:80:00.1'}]
         contents = utils.get_file_data(sriov_config._SRIOV_CONFIG_FILE)
 
         vf_map = yaml.safe_load(contents) if contents else []
@@ -441,6 +483,36 @@ class TestUtils(base.TestCase):
                           utils.bind_dpdk_interfaces, 'eth2', 'vfio-pci',
                           False)
 
+    def test_bind_dpdk_interfaces_same_driver(self):
+        mocked_open = mock.mock_open(read_data='DRIVER=vfio-pci\n')
+        self.stub_out('os_net_config.utils.open', mocked_open)
+        mocked_logger = mock.Mock()
+        self.stub_out('os_net_config.utils.logger.info', mocked_logger)
+        try:
+            utils.bind_dpdk_interfaces('eth1', 'vfio-pci', False)
+        except utils.OvsDpdkBindException:
+            self.fail("Received OvsDpdkBindException unexpectedly")
+        msg = "Driver (vfio-pci) is already bound to the device (eth1)"
+        mocked_logger.assert_called_with(msg)
+
+    def test_get_interface_driver(self):
+        mocked_open = mock.mock_open(read_data='DRIVER=vfio-pci\n')
+        self.stub_out('os_net_config.utils.open', mocked_open)
+        driver = utils.get_interface_driver('eth1')
+        self.assertEqual(driver, 'vfio-pci')
+
+    def test_get_interface_driver_fail_none(self):
+        mocked_open = mock.mock_open(read_data='')
+        self.stub_out('os_net_config.utils.open', mocked_open)
+        driver = utils.get_interface_driver('eth1')
+        self.assertFalse(driver)
+
+    def test_get_interface_driver_fail_empty(self):
+        mocked_open = mock.mock_open(read_data='DRIVER\n')
+        self.stub_out('os_net_config.utils.open', mocked_open)
+        driver = utils.get_interface_driver('eth1')
+        self.assertFalse(driver)
+
     def test__update_dpdk_map_new(self):
         utils._update_dpdk_map('eth1', '0000:03:00.0', '01:02:03:04:05:06',
                                'vfio-pci')
@@ -524,6 +596,44 @@ class TestUtils(base.TestCase):
     def test_interface_mac_raises(self):
         self.assertRaises(IOError, utils.interface_mac, 'ens20f2p3')
 
+    def test_get_dpdk_devargs_mlnx(self):
+        def test_execute(name, dummy1, dummy2=None, dummy3=None):
+            if 'ethtool' in name:
+                out = _PCI_OUTPUT
+                return out, None
+
+        def test_get_stored_pci_address(ifname, noop):
+            return "0000:00:07.0"
+
+        self.stub_out('oslo_concurrency.processutils.execute', test_execute)
+        self.stub_out('os_net_config.utils.get_stored_pci_address',
+                      test_get_stored_pci_address)
+        tmpdir = tempfile.mkdtemp()
+        self.stub_out('os_net_config.utils._SYS_CLASS_NET', tmpdir)
+        nic = 'p4p1'
+        nic_path = os.path.join(tmpdir, nic)
+        os.makedirs(nic_path)
+        os.makedirs(os.path.join(nic_path, 'device'))
+        # Testing standard Mellanox Connect-X cards
+        with open(os.path.join(nic_path, 'operstate'), 'w') as f:
+            f.write('up')
+        with open(os.path.join(nic_path, 'address'), 'w') as f:
+            f.write('00:0f:21:69:39:14')
+        with open(os.path.join(nic_path, 'device', 'vendor'), 'w') as f:
+            f.write('0x15b3')
+        self.assertEqual(utils.get_dpdk_devargs(nic, False),
+                         '0000:00:19.0')
+        # now testing the Mellanox CX3
+        with open(os.path.join(nic_path, 'device', 'device'), 'w') as f:
+            f.write('0x1007')
+        self.assertEqual(utils.get_dpdk_devargs(nic, False),
+                         'class=eth,mac=00:0f:21:69:39:14')
+        with open(os.path.join(nic_path, 'device', 'vendor'), 'w') as f:
+            f.write('0x15b4')
+        self.assertEqual(utils.get_dpdk_devargs(nic, False),
+                         '0000:00:07.0')
+        shutil.rmtree(tmpdir)
+
     def test_is_active_nic_for_sriov_vf(self):
 
         tmpdir = tempfile.mkdtemp()
