@@ -105,14 +105,14 @@ def get_file_data(filename):
 
 def interface_mac(name):
     try:  # If the iface is part of a Linux bond, the real MAC is only here.
-        with open('/sys/class/net/%s/bonding_slave/perm_hwaddr' % name,
+        with open(_SYS_CLASS_NET + '/%s/bonding_slave/perm_hwaddr' % name,
                   'r') as f:
             return f.read().rstrip()
     except IOError:
         pass  # Iface is not part of a bond, continue
 
     try:
-        with open('/sys/class/net/%s/address' % name, 'r') as f:
+        with open(_SYS_CLASS_NET + '/%s/address' % name, 'r') as f:
             return f.read().rstrip()
     except IOError:
         # If the interface is bound to a DPDK driver, get the mac address from
@@ -251,6 +251,11 @@ def diff(filename, data):
 
 
 def bind_dpdk_interfaces(ifname, driver, noop):
+    iface_driver = get_interface_driver(ifname)
+    if iface_driver == driver:
+        logger.info("Driver (%s) is already bound to the device (%s)" %
+                    (driver, ifname))
+        return
     pci_address = get_pci_address(ifname, noop)
     if not noop:
         if pci_address:
@@ -362,17 +367,41 @@ def get_device_id(ifname):
         return
 
 
+def get_interface_driver(ifname):
+    try:
+        uevent = '%s/%s/device/uevent' % (_SYS_CLASS_NET, ifname)
+        with open(uevent, 'r') as f:
+            out = f.read().strip()
+            for line in out.split('\n'):
+                if 'DRIVER' in line:
+                    driver = line.split('=')
+                    if len(driver) == 2:
+                        return driver[1]
+    except IOError:
+        return
+
+
 def get_dpdk_devargs(ifname, noop):
     if not noop:
         vendor_id = get_vendor_id(ifname)
         device_id = get_device_id(ifname)
-        if vendor_id == "0x15b3" and device_id == "0x1007":
-            # Some NICs (i.e. Mellanox ConnectX-3) have only one PCI address
-            # associated with multiple ports. Using a PCI device won’t work.
-            # Instead, we should use "class=eth,mac=<MAC>"
-            dpdk_devargs = "class=eth,mac=%s" % interface_mac(ifname)
+        if vendor_id == "0x15b3":
+            logger.info("Getting devargs for Mellanox cards")
+            if device_id == "0x1007":
+                # Some NICs (i.e. Mellanox ConnectX-3) have only one PCI
+                # address associated with multiple ports. Using a PCI
+                # device won’t work. Instead, we should use
+                # "class=eth,mac=<MAC>"
+                dpdk_devargs = "class=eth,mac=%s" % interface_mac(ifname)
+            elif is_active_nic(ifname):
+                # Other Mellanox devices are active and they are not stored
+                # in dpdk_mapping.yaml file, so we need to get their pci
+                # address with ethtool.
+                dpdk_devargs = get_pci_address(ifname, noop)
         else:
+            logger.info("Getting stored PCI address as devarg")
             dpdk_devargs = get_stored_pci_address(ifname, noop)
+        logger.debug("Devargs found: %s" % (dpdk_devargs))
         return dpdk_devargs
 
 
@@ -417,6 +446,10 @@ def _get_dpdk_mac_address(name):
 def update_sriov_pf_map(ifname, numvfs, noop, promisc=None,
                         link_mode='legacy'):
     if not noop:
+        cur_numvfs = sriov_config.get_numvfs(ifname)
+        if cur_numvfs > 0 and cur_numvfs != numvfs:
+            msg = ("Can't change the numvfs for %s" % ifname)
+            raise sriov_config.SRIOVNumvfsException(msg)
         sriov_map = _get_sriov_map()
         for item in sriov_map:
             if item['device_type'] == 'pf' and item['name'] == ifname:
@@ -445,7 +478,7 @@ def _get_sriov_map():
 
 
 def _set_vf_fields(vf_name, vlan_id, qos, spoofcheck, trust, state, macaddr,
-                   promisc):
+                   promisc, pci_address, min_tx_rate, max_tx_rate):
     vf_configs = {}
     vf_configs['name'] = vf_name
     if vlan_id != 0:
@@ -456,11 +489,14 @@ def _set_vf_fields(vf_name, vlan_id, qos, spoofcheck, trust, state, macaddr,
         vf_configs['qos'] = qos
     else:
         vf_configs['qos'] = None
+    vf_configs['min_tx_rate'] = min_tx_rate
+    vf_configs['max_tx_rate'] = max_tx_rate
     vf_configs['spoofcheck'] = spoofcheck
     vf_configs['trust'] = trust
     vf_configs['state'] = state
     vf_configs['macaddr'] = macaddr
     vf_configs['promisc'] = promisc
+    vf_configs['pci_address'] = pci_address
     return vf_configs
 
 
@@ -472,14 +508,16 @@ def _clear_empty_values(vf_config):
 
 def update_sriov_vf_map(pf_name, vfid, vf_name, vlan_id=0, qos=0,
                         spoofcheck=None, trust=None, state=None, macaddr=None,
-                        promisc=None):
+                        promisc=None, pci_address=None,
+                        min_tx_rate=0, max_tx_rate=0):
     sriov_map = _get_sriov_map()
     for item in sriov_map:
         if (item['device_type'] == 'vf' and
            item['device'].get('name') == pf_name and
            item['device'].get('vfid') == vfid):
             item.update(_set_vf_fields(vf_name, vlan_id, qos, spoofcheck,
-                                       trust, state, macaddr, promisc))
+                                       trust, state, macaddr, promisc,
+                                       pci_address, min_tx_rate, max_tx_rate))
             _clear_empty_values(item)
             break
     else:
@@ -487,7 +525,8 @@ def update_sriov_vf_map(pf_name, vfid, vf_name, vlan_id=0, qos=0,
         new_item['device_type'] = 'vf'
         new_item['device'] = {"name": pf_name, "vfid": vfid}
         new_item.update(_set_vf_fields(vf_name, vlan_id, qos, spoofcheck,
-                                       trust, state, macaddr, promisc))
+                                       trust, state, macaddr, promisc,
+                                       pci_address, min_tx_rate, max_tx_rate))
         _clear_empty_values(new_item)
         sriov_map.append(new_item)
 
@@ -514,9 +553,10 @@ def _configure_sriov_config_service():
     processutils.execute('systemctl', 'enable', 'sriov_config')
 
 
-def configure_sriov_pfs():
+def configure_sriov_pfs(execution_from_cli=False, restart_openvswitch=False):
     logger.info("Configuring PFs now")
-    sriov_config.configure_sriov_pf()
+    sriov_config.configure_sriov_pf(execution_from_cli=execution_from_cli,
+                                    restart_openvswitch=restart_openvswitch)
     _configure_sriov_config_service()
 
 
